/*
 * netstat.c - Version 0.5
 *
 * Copyright (c) 1993  Fred Baumgarten (dc6iq@insu1.etec.uni-karlsruhe.de)
 *
 */

#define Version "Version 0.5 (c) 1993, Fred Baumgarten <dc6iq@insu1.etec.uni-karlsruhe.de>"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <linux/route.h>
#include <linux/tcp.h>
#include <strings.h>

#define _PATH_PROCNET_TCP "/proc/net/tcp"
#define _PATH_PROCNET_UDP "/proc/net/udp"
#define _PATH_PROCNET_RAW "/proc/net/raw"
#define _PATH_PROCNET_ROUTE "/proc/net/route"

#define E_READ -1
#define E_PARA -2

FILE *procinfo;
char *line[2000];

int flag_cnt = 0;
int flag_deb = 0;
int flag_not = 0;
int flag_opt = 0;
int flag_raw = 0;
int flag_rou = 0;
int flag_tcp = 0;
int flag_udp = 0;

char *tcp_name[10000], *udp_name[18000], *raw_name[1000];

struct numbtoname {
  unsigned long addr;
  char *name;
  struct numbtoname *next;
};

#define NULLNN (struct numbtoname *) 0

struct numbtoname *nn = NULLNN;

void query_host(char *h, unsigned long ad) {
  struct hostent *ent = (struct hostent *) 0;
  struct numbtoname *pn = NULLNN;

  if (!ad) {
    strcpy(h, "*");
  } else {
    pn = nn;
    *h = '\0';
    while (pn != NULLNN) {
      if (pn->addr == ad) {
	strcpy(h, pn->name);
	break;
      }
      pn = pn->next;
    }
    if (*h == '\0') {
      if (!flag_not) ent = gethostbyaddr((char *) &ad, 4, AF_INET);
      if (ent) {
	strcpy(h, ent->h_name);
      } else {
	sprintf(h, "%d.%d.%d.%d", (int) (ad & 0xff), (int) ((ad >> 8) & 0xff),
		(int) ((ad >> 16) & 0xff), (int) ((ad >> 24) & 0xff));
      }
      pn = (struct numbtoname *)malloc(sizeof(struct numbtoname));
      pn->addr = ad;
      pn->next = nn;
      pn->name = malloc(strlen(h)+1);
      strcpy(pn->name, h);
      nn = pn;
    }
  }
}

int read_services(void) {
  char buffer[2048], name[32], protocol[16], dummy[1024];
  int number;
  FILE *serv;

  serv = fopen(_PATH_SERVICES, "r");
  if (!serv) {
    perror(_PATH_SERVICES);
    return E_READ;
  }

  while (!feof(serv)) {
    fgets(buffer, 2047, serv);
    if (sscanf(buffer, "%s%d/%3s%s\n", name, &number, protocol, dummy) >= 3) {
      if (!strcmp(protocol, "tcp")) {
	tcp_name[number] = malloc(strlen(name)+1);
	if (tcp_name[number]) {
	  strcpy(tcp_name[number], name);
	}
      } else {
	if (!strcmp(protocol, "udp")) {
	  udp_name[number] = malloc(strlen(name)+1);
	  if (udp_name[number]) {
	    strcpy(udp_name[number], name);
	  } else {
	    printf("malloc error\n");
	  }
	} else {
	  if (!strcmp(protocol, "raw")) {
	    raw_name[number] = malloc(strlen(name)+1);
	    if (raw_name[number]) {
	      strcpy(raw_name[number], name);
	    } else {
	      printf("malloc error\n");
	    }
	  }
	}
      }
    }
  }
  fclose(serv);
  return 0;
}

char *get_sname(int socknumber, char *proto) {
  static char buffer[64];

  if (flag_not) {
    sprintf(buffer, "%d", socknumber);
    return buffer;
  }
  if (socknumber == 0)
    return "*";
  if (!strcmp(proto, "tcp")) {
    if (tcp_name[socknumber]) {
      sprintf(buffer, "%s", tcp_name[socknumber]);
    } else {
      sprintf(buffer, "%d", socknumber);
    }
  }
  if (!strcmp(proto, "udp")) {
    if (udp_name[socknumber]) {
      sprintf(buffer, "%s", udp_name[socknumber]);
    } else {
      sprintf(buffer, "%d", socknumber);
    }
  }
  if (!strcmp(proto, "raw")) {
    if (raw_name[socknumber]) {
      sprintf(buffer, "%s", raw_name[socknumber]);
    } else {
      sprintf(buffer, "%d", socknumber);
    }
  }
  return buffer;
}

int route_info(void) {
  char buffer[512], iface[16], net_addr[64], gate_addr[64], flags[16];
  int anz, iflags, refcnt, use, metric, zeile = 0;
  unsigned long net, gate;

  printf("Kernel routing table\n");
  printf("Destination net/address   Gateway address           Flags RefCnt    Use Iface\n");
  procinfo = fopen(_PATH_PROCNET_ROUTE, "r");
  if (!procinfo) {
    perror(_PATH_PROCNET_ROUTE);
    return E_READ;
  }
  fgets(buffer, 1023, procinfo);
  while (!feof(procinfo)) {
    fgets(buffer, 1023, procinfo);
    if ((line[zeile] = (char *)malloc(strlen(buffer)+1)) != NULL) {
      strcpy(line[zeile++], buffer);
      if (flag_deb) {
        fprintf(stderr, "%s", buffer);
      }
    }
  }
  fclose(procinfo);
  zeile--;
  while (zeile>=0) {
    anz = sscanf(line[zeile--], "%s %lX %lX %X %d %d %d\n",
	   iface, &net, &gate, &iflags, &refcnt, &use, &metric);
    if (anz == 7) {
      query_host(net_addr, net);
      net_addr[25]='\0';
      if (net == 0L) strcpy(net_addr, "default");
      query_host(gate_addr, gate);
      gate_addr[25]='\0';
      flags[0]='\0';
      if (iflags & RTF_UP) strcat(flags, "U");
      if (iflags & RTF_GATEWAY) strcat(flags, "G");
      if (iflags & RTF_HOST) strcat(flags, "H");
      if (iflags & RTF_REINSTATE) strcat(flags, "R");
      if (iflags & RTF_DYNAMIC) strcat(flags, "D");
      if (iflags & RTF_MODIFIED) strcat(flags, "M");
      printf("%-25s %-25s %-5s %6d %6d %s\n", net_addr, gate_addr, flags, refcnt, use, iface);
    }
  }
  return 0;
}

int tcp_info(void) {
  char buffer[1024], local_addr[64], rem_addr[64], *tcp_state, timers[512];
  int anz, local_port, rem_port, d, state, zeile = 0;
  unsigned long rxq, txq, localaddr, remaddr;
  
  procinfo = fopen(_PATH_PROCNET_TCP, "r");
  if (!procinfo) {
    perror(_PATH_PROCNET_TCP);
    return E_READ;
  }
  fgets(buffer, 1023, procinfo);
  while (!feof(procinfo)) {
    fgets(buffer, 1023, procinfo);
    anz = strlen(buffer)+1;
    if ((line[zeile] = (char *)malloc(strlen(buffer)+1)) != NULL) {
      strcpy(line[zeile++], buffer);
      if (flag_deb) {
        fprintf(stderr, "%s", buffer);
      }
    }
  }
  fclose(procinfo);
  zeile--;
  zeile--;
  while (zeile>=0) {
    timers[0]='\0';
    anz = sscanf(line[zeile--], "%d: %lX:%X %lX:%X %X %X:%X %s\n",
		 &d, &localaddr, &local_port, &remaddr, &rem_port,
		 &state, &txq, &rxq, timers);
    if (!flag_opt) timers[0]='\0';
    if (flag_deb) fprintf(stderr, "%s -> %d args", line[zeile+1], anz);
    if (anz >= 8) {
      if (state != TCP_ESTABLISHED) {
        txq=0;		/* Here are stupid thigs... Why isnt it zero ? */
      }
      switch (state) {
      case TCP_ESTABLISHED:
	tcp_state = "ESTABLISHED";
	rxq--;
	break;
      case TCP_SYN_SENT:
	tcp_state = "SYN_SENT";
	break;
      case TCP_SYN_RECV:
	tcp_state = "SYN_RECV";
	break;
      case TCP_FIN_WAIT1:
	tcp_state = "FIN_WAIT1";
	break;
      case TCP_FIN_WAIT2:
	tcp_state = "FIN_WAIT2";
	break;
      case TCP_TIME_WAIT:
	tcp_state = "TIME_WAIT";
	break;
      case TCP_CLOSE:
	tcp_state = "CLOSE";
	break;
      case TCP_CLOSE_WAIT:
	tcp_state = "CLOSE_WAIT";
	break;
      case TCP_LAST_ACK:
	tcp_state = "LAST_ACK";
	break;
      case TCP_LISTEN:
	tcp_state = "LISTEN";
	break;
      default:
	tcp_state = "?? ()";
	break;
      }
      query_host(local_addr, localaddr);
      query_host(rem_addr, remaddr);
      if (flag_tcp || rem_port) {
	sprintf(buffer, "%s",get_sname(local_port, "tcp"));
	if ((strlen(local_addr) + strlen(buffer)) >21) {
	  local_addr[22-strlen(buffer)] = '\0';
	}
	strcat(local_addr, ":");
	strcat(local_addr, buffer);
	sprintf(buffer, "%s",get_sname(rem_port, "tcp"));
	if ((strlen(rem_addr) + strlen(buffer)) >21) {
	  rem_addr[22-strlen(buffer)] = '\0';
	}
	strcat(rem_addr, ":");
	strcat(rem_addr, buffer);
	printf("tcp   %6d %6d %-22s %-22s %s %s\n", rxq, txq, local_addr,
	       rem_addr, tcp_state, timers);
      }
    }
  }
  return 0;
}

int udp_info(void) {
  char buffer[1024], local_addr[64], rem_addr[64], *udp_state, timers[512];
  int anz, local_port, rem_port, d, state, zeile = 0;
  unsigned long rxq, txq, localaddr, remaddr;
  
  procinfo = fopen(_PATH_PROCNET_UDP, "r");
  if (!procinfo) {
    perror(_PATH_PROCNET_UDP);
    return E_READ;
  }
  fgets(buffer, 1023, procinfo);
  while (!feof(procinfo)) {
    fgets(buffer, 1023, procinfo);
    if ((line[zeile] = (char *)malloc(strlen(buffer)+1)) != NULL) {
      strcpy(line[zeile++], buffer);
      if (flag_deb) {
        fprintf(stderr, "%s", buffer);
      }
    }
  }
  fclose(procinfo);
  zeile--;
  zeile--;
  while (zeile>=0) {
    timers[0]='\0';
    anz = sscanf(line[zeile--], "%d: %lX:%X %lX:%X %X %X:%X %s\n",
		 &d, &localaddr, &local_port, &remaddr, &rem_port,
		 &state, &txq, &rxq, timers);
    if (!flag_opt) timers[0]='\0';
    if (flag_deb) fprintf(stderr, "%s -> %d args", line[zeile+1], anz);
    if (anz >= 5) {
      if (state != TCP_ESTABLISHED) {
        txq=0;		/* Here are stupid thigs... Why isnt it zero ? */
      }
      switch (state) {
      case TCP_ESTABLISHED:
	udp_state = "ESTABLISHED";
	rxq--;
	break;
      default:
	udp_state = "";
	break;
      }
      query_host(local_addr, localaddr);
      query_host(rem_addr, remaddr);
      if (flag_udp) {
	sprintf(buffer, "%s",get_sname(local_port, "udp"));
	if ((strlen(local_addr) + strlen(buffer)) >22) {
	  local_addr[23-strlen(buffer)] = '\0';
	}
	strcat(local_addr, ":");
	strcat(local_addr, buffer);
	sprintf(buffer, "%s",get_sname(rem_port, "udp"));
	if ((strlen(rem_addr) + strlen(buffer)) >22) {
	  rem_addr[23-strlen(buffer)] = '\0';
	}
	strcat(rem_addr, ":");
	strcat(rem_addr, buffer);
	printf("udp   %6d %6d %-22s %-22s %s %s\n", rxq, txq, local_addr,
	       rem_addr, udp_state, timers);
      }
    }
  }
  return 0;
}

int raw_info(void) {
  char buffer[1024], local_addr[64], rem_addr[64], *raw_state, timers[512];
  int anz, local_port, rem_port, d, state, zeile = 0;
  unsigned long rxq, txq, localaddr, remaddr;
  
  procinfo = fopen(_PATH_PROCNET_RAW, "r");
  if (!procinfo) {
    perror(_PATH_PROCNET_RAW);
    return E_READ;
  }
  fgets(buffer, 1023, procinfo);
  while (!feof(procinfo)) {
    fgets(buffer, 1023, procinfo);
    if ((line[zeile] = (char *)malloc(strlen(buffer)+1)) != NULL) {
      strcpy(line[zeile++], buffer);
      if (flag_deb) {
        fprintf(stderr, "%s", buffer);
      }
    }
  }
  fclose(procinfo);
  zeile--;
  zeile--;
  while (zeile>=0) {
    timers[0]='\0';
    anz = sscanf(line[zeile--], "%d: %lX:%X %lX:%X %X %X:%X %s\n",
		 &d, &localaddr, &local_port, &remaddr, &rem_port,
		 &state, &txq, &rxq, timers);
    if (!flag_opt) timers[0]='\0';
    if (flag_deb) fprintf(stderr, "%s -> %d args", line[zeile+1], anz);
    if (anz >= 5) {
      raw_state = "";
      txq=0;		/* Here are stupid thigs... Why isnt it zero ? */
      query_host(local_addr, localaddr);
      query_host(rem_addr, remaddr);
      if (flag_raw) {
	sprintf(buffer, "%s",get_sname(local_port, "raw"));
	if ((strlen(local_addr) + strlen(buffer)) >22) {
	  local_addr[23-strlen(buffer)] = '\0';
	}
	strcat(local_addr, ":");
	strcat(local_addr, buffer);
	sprintf(buffer, "%s",get_sname(rem_port, "raw"));
	if ((strlen(rem_addr) + strlen(buffer)) >22) {
	  rem_addr[23-strlen(buffer)] = '\0';
	}
	strcat(rem_addr, ":");
	strcat(rem_addr, buffer);
	printf("raw   %6d %6d %-22s %-22s %s %s\n", rxq, txq, local_addr,
	       rem_addr, raw_state, timers);
      }
    }
  }
  return 0;
}

void usage(void) {
  fprintf(stderr, "Usage:\tnetstat [options]\n");
  fprintf(stderr, "\t-a all sockets (tcp+udp)\n");
  fprintf(stderr, "\t-c continous listing\n");
  fprintf(stderr, "\t-n show network numbers instead of names\n");
  fprintf(stderr, "\t-r show kernel routing table\n");
  fprintf(stderr, "\t-t show all tcp connections\n");
  fprintf(stderr, "\t-u show all udp ports\n");
  fprintf(stderr, "\t-w show all raw ports\n");
}

int main (int argc, char *argv[]) {
  int erg;
  char c;

  while ((c = getopt(argc, argv, "acdnortuvw")) != EOF)
    switch(c) {
    case 'a':
      flag_tcp++;
      flag_udp++;
      flag_raw++;
      break;
    case 'c':
      flag_cnt++;
      break;
    case 'd':
      flag_deb++;
      break;
    case 'n':
      flag_not++;
      break;
    case 'o':
      flag_opt++;
      break;
    case 'r':
      flag_rou++;
      break;
    case 't':
      flag_tcp++;
      break;
    case 'u':
      flag_udp++;
      break;
    case 'v':
      printf("netstat.c %s\n", Version);
      return 0;
      break;
    case 'w':
      flag_raw++;
      break;
    case '?':
      usage();
      return E_PARA;
    }
  
  if (flag_rou) {
    for (;; ) {
      erg = route_info();
      if (!flag_cnt || erg) break;
      sleep(1);
    }
    return erg;
  }
  
  if ((erg = read_services()) != 0) return erg;

  for (;; ) {
    printf("Active Internet connections\n");
    printf("Proto Recv-Q Send-Q Local Address          Foreign Address        (State)\n");
    if ((!flag_udp && !flag_raw) || flag_tcp) {
      erg = tcp_info();
      if (erg) return erg;
    }
    if (flag_udp) {
      erg = udp_info();
      if (erg) return erg;
    }
    if (flag_raw) {
      erg = raw_info();
      if (erg) return erg;
    }
    if (!flag_cnt || erg) break;
    sleep(1);
  }
  return erg;
}
